@@ -1,3 +1,5 @@
+# Copyright (C) 2012-2013 Steven Myint
+
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
 # "Software"), to deal in the Software without restriction, including
@@ -16,17 +18,25 @@
 # CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
 # TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
 # SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+
 """Formats docstrings to follow PEP 257."""
 
+from __future__ import print_function
+
+import errno
+import io
+import os
 import re
 import tokenize
+
+
+__version__ = '0.2.7'
+
+
 try:
-    from StringIO import StringIO
-except ImportError:
-    from io import StringIO
-
-
-__version__ = '0.2.6'
+    unicode
+except NameError:
+    unicode = str
 
 
 def format_code(source,
@@ -38,7 +48,7 @@ def format_code(source,
     Wrap summary lines if summary_wrap_length is greater than 0.
 
     """
-    sio = StringIO(source)
+    sio = io.StringIO(source)
     formatted = ''
     previous_token_string = ''
     previous_token_type = None
@@ -134,14 +144,15 @@ def format_docstring(indentation, docstring,
 
 {description}{post_description}
 {indentation}"""\
-'''.format(pre_summary=('\n' + indentation if pre_summary_newline else ''),
+'''.format(pre_summary=('\n' + indentation if pre_summary_newline
+                        else unicode()),
            summary=wrap_summary(normalize_summary(summary),
                                 wrap_length=summary_wrap_length,
                                 initial_indent=initial_indent,
                                 subsequent_indent=indentation).lstrip(),
            description='\n'.join([indent_non_indented(l, indentation).rstrip()
                                   for l in description.splitlines()]),
-           post_description=('\n' if post_description_blank else ''),
+           post_description=('\n' if post_description_blank else unicode()),
            indentation=indentation)
     else:
         return wrap_summary('"""' + normalize_summary(contents) + '"""',
@@ -177,7 +188,7 @@ def split_summary_and_description(contents):
         if len(split) == 2:
             return (split[0].strip() + '.', split[1].strip())
         else:
-            return (split[0].strip(), '')
+            return (split[0].strip(), unicode())
 
 
 def strip_docstring(docstring):
@@ -217,7 +228,6 @@ def wrap_summary(summary, initial_indent, subsequent_indent, wrap_length):
 
 def open_with_encoding(filename, encoding, mode='r'):
     """Return opened file with a specific encoding."""
-    import io
     return io.open(filename, mode=mode, encoding=encoding,
                    newline='')  # Preserve line endings
 
@@ -238,14 +248,42 @@ def detect_encoding(filename):
         return 'latin-1'
 
 
-def main(argv, standard_out):
+def format_file(filename, args, standard_out):
+    """Run format_code() on a file."""
+    encoding = detect_encoding(filename)
+    with open_with_encoding(filename, encoding=encoding) as input_file:
+        source = input_file.read()
+        formatted_source = format_code(
+            source,
+            summary_wrap_length=args.wrap_summaries,
+            pre_summary_newline=args.pre_summary_newline,
+            post_description_blank=args.post_description_blank)
+
+    if source != formatted_source:
+        if args.in_place:
+            with open_with_encoding(filename, mode='w',
+                                    encoding=encoding) as output_file:
+                output_file.write(formatted_source)
+        else:
+            import difflib
+            diff = difflib.unified_diff(
+                io.StringIO(source).readlines(),
+                io.StringIO(formatted_source).readlines(),
+                'before/' + filename,
+                'after/' + filename)
+            standard_out.write(unicode().join(diff))
+
+
+def main(argv, standard_out, standard_error):
     """Main entry point."""
     import argparse
     parser = argparse.ArgumentParser(description=__doc__, prog='docformatter')
-    parser.add_argument('--in-place', action='store_true',
+    parser.add_argument('-i', '--in-place', action='store_true',
                         help='make changes to files instead of printing diffs')
+    parser.add_argument('-r', '--recursive', action='store_true',
+                        help='drill down directories recursively')
     parser.add_argument(
-        '--wrap-long-summaries', default=0, type=int, metavar='LENGTH',
+        '--wrap-summaries', default=79, type=int, metavar='length',
         help='wrap long summary lines at this length (default: %(default)s)')
     parser.add_argument('--no-blank', dest='post_description_blank',
                         action='store_false',
@@ -254,32 +292,28 @@ def main(argv, standard_out):
                         action='store_true',
                         help='add a newline before the summary of a '
                              'multi-line docstring')
-    parser.add_argument('--version', action='version', version=__version__)
+    parser.add_argument('--version', action='version',
+                        version='%(prog)s ' + __version__)
     parser.add_argument('files', nargs='+',
                         help='files to format')
 
     args = parser.parse_args(argv[1:])
 
-    for filename in args.files:
-        encoding = detect_encoding(filename)
-        with open_with_encoding(filename, encoding=encoding) as input_file:
-            source = input_file.read()
-            formatted_source = format_code(
-                source,
-                summary_wrap_length=args.wrap_long_summaries,
-                pre_summary_newline=args.pre_summary_newline,
-                post_description_blank=args.post_description_blank)
-
-        if source != formatted_source:
-            if args.in_place:
-                with open_with_encoding(filename, mode='w',
-                                        encoding=encoding) as output_file:
-                    output_file.write(formatted_source)
-            else:
-                import difflib
-                diff = difflib.unified_diff(
-                    StringIO(source).readlines(),
-                    StringIO(formatted_source).readlines(),
-                    'before/' + filename,
-                    'after/' + filename)
-                standard_out.write(''.join(diff))
+    filenames = list(set(args.files))
+    while filenames:
+        name = filenames.pop(0)
+        if args.recursive and os.path.isdir(name):
+            for root, directories, children in os.walk(name):
+                filenames += [os.path.join(root, f) for f in children
+                              if f.endswith('.py') and
+                              not f.startswith('.')]
+                for d in directories:
+                    if d.startswith('.'):
+                        directories.remove(d)
+        else:
+            try:
+                format_file(name, args=args, standard_out=standard_out)
+            except IOError as exception:
+                if exception.errno == errno.EPIPE:
+                    return
+                print(exception, file=standard_error)
